const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const Business = db.businesses
const DataOption = db.dataOptions
const UserDetail = db.userDetails
const Country = db.countries
const Region = db.regions
const City = db.cities
const Barangay = db.barangays

// Create
const add = async (req, res) => {
 try {
  let info = {
    id: uuidv4(),
    name: req.body.name,
    sequence: req.body.sequence,
    description: req.body.description,
    longintude: req.body.longintude,
    latitude: req.body.latitude,
    disabled: req.body.disabled,
    contact: req.body.contact,
    ownerId: req.body.ownerId
   }
 
  await Business.create(info)
    .then((business) => {
      return res.status(200).send(business)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Create and Upload
const addAndUpload = async (req, res) => {
  try {
   const filePath = req.body.imagePath

   let info = {
     id: uuidv4(),
     name: req.body.name,
     sequence: req.body.sequence,
     description: req.body.description,
     longintude: req.body.longintude,
     latitude: req.body.latitude,
     disabled: req.body.disabled,
     contact: req.body.contact,
     ownerId: req.body.ownerId,
     approved: req.body.approved && req.body.approved !== undefined ? req.body.approved : false,
     approvedBy: req.body.approvedBy,
     approvedOn: req.body.approvedOn,
     attachments: filePath
    }
  
   await Business.create(info)
     .then((business) => {
       return res.status(200).send(business)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await Business.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'owner', nested: true, include: [
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await Business.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: [
    { model: UserDetail, as: 'owner', nested: true, include: [
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((dataOptions) => {
    return res.status(200).send(dataOptions)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let userFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           if ( prop === 'owner') {
            userFilter[innerprop] = req.body.filter[prop][innerprop]
           }
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
  
  await Business.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'owner', nested: true, where: { ...userFilter }, include: [
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, userFilter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let businessId = req.params.id

  if (!businessId) return res.status(400).send({ statusCode: 400, message: 'Invalid Business ID' })

  await Business.findOne({
   where: { id: businessId },
   include: [
    { model: UserDetail, as: 'owner', nested: true, include: [
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((business) => {
    return res.status(200).send(business)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let businessId = req.params.id

  if (!businessId) return res.status(400).send({ statusCode: 400, message: 'Invalid Business ID' })

  let info = {
    name: req.body.name,
    sequence: req.body.sequence,
    description: req.body.description,
    longintude: req.body.longintude,
    latitude: req.body.latitude,
    disabled: req.body.disabled,
    contact: req.body.contact,
    ownerId: req.body.ownerId,
    approved: req.body.approved && req.body.approved !== undefined ? req.body.approved : false,
    approvedBy: req.body.approvedBy,
    approvedOn: req.body.approvedOn
  }

  await Business.update(info, { where: { id: businessId } })
  .then((business) => {
    return res.status(200).send(business)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let businessId = req.params.id

  if (!businessId) return res.status(400).send({ statusCode: 400, message: 'Invalid Business ID' })

  await Business.destroy({
   where: {id: businessId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

const resolveBusiness = async (req, res) => {4
  try {
    let businessId = req.params.id || req.body.id

    if (!businessId) return res.status(400).send({ statusCode: 400, message: 'Invalid Business ID' })

    await Business.update({
      approved: true,
      approvedBy: req.body.approvedBy,
      approvedOn: req.body.approvedOn
    }, { where: { id: businessId } })
    .then((business) => {
     return res.status(200).send(business)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

const rejectBusiness = async (req, res) => {
  try {
    let businessId = req.params.id || req.body.id

    if (!businessId) return res.status(400).send({ statusCode: 400, message: 'Invalid Business ID' })

    await Business.update({
     rejected: true,
     rejectionNotes: req.body.rejectionNotes,
     rejectedBy: req.body.rejectedBy
    }, { where: { id: businessId } })
    .then((business) => {
     return res.status(200).send(business)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

// Return with count
const returnWithCount = async (res, filter, userFilter, response, limit, isAdmin) => {
  if (isAdmin) {
    Business.count({
      where: { ...filter },
      include: [ { model: UserDetail, as: 'owner', nested: true, where: { ...userFilter }} ]
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    Business.count({
      where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      },
      include: [ { model: UserDetail, as: 'owner', nested: true, where: { ...userFilter }} ]
      }).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
  
}

module.exports = {
  add,
  addAndUpload,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  rejectBusiness,
  resolveBusiness,
  returnWithCount
}