const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const moment = require('moment')

const mailController = require('./mail.controller')
const smsController = require('./sms.controller')

const CarSticker = db.carStickers
const UserDetails = db.userDetails
const User = db.users

// Request Document - Certificate with Cedula Photo
const createRequestCarSticker = async (req, res) => {
  try {
    let orcrPath = req.body.orcrPath
    let vehiclePhotoPath = req.body.vehiclePhotoPath

    let request = {
      id: uuidv4(),
      name: req.body.name,
      address: req.body.address,
      contactNumber: req.body.contactNumber,
      otherDrivers: req.body.otherDrivers,
      vehiclePlateNumber: req.body.vehiclePlateNumber,
      vehicleModel: req.body.vehicleModel,
      requestedById: req.body.requestedById,
      orcr: orcrPath,
      vehiclePhoto: vehiclePhotoPath,
      vehicleConductionStickerNumber: null,
      status: req.body.status,
      statusUpdatedBy: req.body.statusUpdatedBy,
      statusUpdatedOn: new Date(),
      statusUpdatedNotes: req.body.statusUpdatedNotes
    }

    log.info('Start saving request car sticker...')

    await CarSticker.create(request)
      .then((data) => {
        CarSticker.findOne({
          where: { id: data.dataValues.id },
          include: [
            { model: UserDetails, as: 'carStickerBy', nested: true,
              include: [
                { model: User, as: 'user', nested: true }
              ]
            }
          ]
        }).then((response) => {
          mailController.sendCarStickerInitialEmail(response)
        })

        return res.status(200).send(data.dataValues)
      }).catch((error) => {
        log.fatal('Could not save request car sticker:', error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })
  } catch (error) {
    log.fatal('Error creating request car sticker:', error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Generate Ticket Number
const generateConductionVehicleNumber = () => {
  return new Promise((resolve, reject) => {
    let dateFormat = moment(new Date()).tz('Asia/Manila').format('YYYY')
    const startOfYear = moment().startOf('year').toDate()
    const endOfYear = moment().endOf('year').toDate()

    CarSticker.count({
      where: { [Op.or]: [
        { status: 'APPROVED' },
        { status: 'COMPLETED' }
        ], [Op.and]: [
          { createdAt: { [Op.between]: [startOfYear, endOfYear] } }
        ]
      }
     })
      .then((totalCount) => {
        let outputCount = String(totalCount + 2).padStart(4, '0')
        const tn = `${dateFormat}-${outputCount}`
        log.info('Generating Car Sticker Number', tn)
        resolve(tn)
      }).catch(reject)
  })
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await CarSticker.findAll({
    where: { ...filter },
    include: [{ model: UserDetails, as: 'carStickerBy', nested: true,
      include: [
        { model: User, as: 'user', nested: true }
      ]
    }],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
  try {
    await CarSticker.findAll({
      where: { [Op.and]: [ { deletedAt: null } ] },
      include: [
        { model: UserDetails, as: 'carStickerBy', nested: true,
          include: [
            { model: User, as: 'user', nested: true }
          ]
        }
      ]
    }).then((response) => {
      return res.status(200).send(response)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let innerFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          if (prop === 'requestedById') {
            filter[prop] = req.body.filter[prop]
          } else {
            filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
          }
        }
      }
    }
  }

  console.log('Filter', filter)
  
  await CarSticker.findAll({
    where: { ...filter },
    include: [
      { model: UserDetails, as: 'carStickerBy', nested: true, where: { ...innerFilter },
        include: [
          { model: User, as: 'user', nested: true }
        ]
      }
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
     return returnWithCount(res, filter, innerFilter, response, limit)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let CarStickerId = req.params.id

  if (!CarStickerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Car Sticker ID' })

  await CarSticker.findOne({
    where: { id: CarStickerId },
    include: [
      { model: UserDetails, as: 'carStickerBy', nested: true,
        include: [
          { model: User, as: 'user', nested: true },
        ]
      }
    ]
  }).then((response) => {
    return res.status(200).send(response)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let CarStickerId = req.params.id
  let carStickerStatus = req.body.status
  let info = {
    status: carStickerStatus,
    statusUpdatedBy: req.body.statusUpdatedBy,
    statusUpdatedOn: req.body.statusUpdatedOn,
    statusUpdatedNotes: req.body.statusUpdatedNotes
  }

  if (carStickerStatus === 'APPROVED') {
    info.vehicleConductionStickerNumber = await generateConductionVehicleNumber()
  }

  if (!CarStickerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Car Sticker ID' })

  await CarSticker.update(info, { where: { id: CarStickerId } })
  .then((response) => {
      CarSticker.findOne({
        where: { id: CarStickerId },
        include: [
          { model: UserDetails, as: 'carStickerBy', nested: true,
            include: [
              { model: User, as: 'user', nested: true }
            ]
          }
        ]
      }).then((data) => {
        let converted = JSON.stringify(data)
        let entity = JSON.parse(converted)
        if (carStickerStatus === 'APPROVED') {
          mailController.sendCarStickerReleaseEmail(entity)
          return res.status(200).send(entity)
        } else if (carStickerStatus === 'CANCELLED') {
          mailController.sendCarStickerCancelledEmail(entity)
          return res.status(200).send(entity)
        } else {
          return res.status(200).send(entity)
        }
       })
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
  try {
    let CarStickerId = req.params.id
    if (!CarStickerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Document ID' })
    await CarSticker.destroy({
      where: {id: CarStickerId}
    }).then(() => {
      return res.status(200).send()
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const returnWithCount = async (res, filter, innerFilter, response, limit) => {
 if (Object.keys(innerFilter).length) {
  await CarSticker.count({
   where: { ...filter },
   include: [{ model: UserDetails, as: 'carStickerBy', nested: true, where: { ...innerFilter } }],
  }).then((totalCount) => {
     let result = {
       content: response,
       totalElements: totalCount,
       totalPages: Math.ceil(totalCount/limit),
     }
     return res.status(200).send(result)
  })
 } else {
  await CarSticker.count({
   where: { ...filter }
  }).then((totalCount) => {
     let result = {
       content: response,
       totalElements: totalCount,
       totalPages: Math.ceil(totalCount/limit),
     }
     return res.status(200).send(result)
  })
 }
 
}

module.exports = {
  createRequestCarSticker,
  generateConductionVehicleNumber,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  returnWithCount
}