const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const City = db.cities
const Region = db.regions

// Create
const add = async (req, res) => {
  try {
    let info = {
      id: uuidv4(),
      name: req.body.name,
      code: req.body.code,
      description: req.body.description,
      regionId: req.body.region ? req.body.region.brgyKey : null,
      disabled: req.body.disabled
    }
  
   await City.create(info)
     .then((city) => {
       return res.status(200).send(city)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All
 const getAll = async (req, res) => {
  try {
   let str = req.body && req.body.str
   let sortBy = req.body.sortBy || 'brgyKey'
   let order = req.body.order || 'ASC'
   let limit = req.body.size ? req.body.size : 20
   let pageCount = req.body.page ? req.body.page : 1
   let limitCount = 0 + (pageCount - 1) * limit
   let page = limitCount > 0 ? limitCount : 0
 
   let filter = str ? {
     [Op.or]: [
       { name: { [Op.substring] : '%' + str + '%' } },
       { code: { [Op.substring] : '%' + str + '%' } }
     ]
   } : ''
 
   await City.findAll({
     where: { ...filter },
     include: [{ model: Region, as: 'region', nested: true }],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit, true)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All Enabled
 const get = async (req, res) => {
  try {
   await City.findAll({
    where: {
      [Op.or]: [
        { disabled: null },
        { disabled: false },
        { deletedAt: null }
      ]
    },
    include: [
     {
       model: Region,
       as: 'region'
     }
    ]
   }).then((citys) => {
     return res.status(200).send(citys)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get Filtered (Admin)
 const getFiltered = async (req, res) => {
  try {
    let sortBy = req.body.sortBy || 'brgyKey'
    let order = req.body.order || 'ASC'
    let limit = req.body.size ? req.body.size : 20
    let pageCount = req.body.page ? req.body.page : 1
    let limitCount = 0 + (pageCount - 1) * limit
    let page = limitCount > 0 ? limitCount : 0

   let filter = {}
   let innerFilter = {}
 
   if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
   
   await City.findAll({
     where: { ...filter },
     include: [{ model: Region, as: 'region', nested: true, where: { ...innerFilter } }],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, innerFilter, response, limit, true)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get One
 const getOne = async (req, res) => {
  try {
   let cityId = req.params.id

   if (!cityId) return res.status(400).send({ statusCode: 400, message: 'Invalid City ID' })
 
   await City.findOne({
    where: { id: cityId },
    include: [{ model: Region, as: 'region', nested: true }]
   }).then((city) => {
     return res.status(200).send(city)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Update One
 const update = async (req, res) => {
  try {
   let cityId = req.params.id

   if (!cityId) return res.status(400).send({ statusCode: 400, message: 'Invalid City ID' })

   let info = {
    name: req.body.name,
    code: req.body.code,
    description: req.body.description,
    regionId: req.body.region ? req.body.region.brgyKey : null,
    disabled: req.body.disabled
  }
 
   await City.update(info, { where: { id: cityId } })
   .then((city) => {
     return res.status(200).send(city)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Delete One
 const remove = async (req, res) => {
  try {
   let cityId = req.params.id

   if (!cityId) return res.status(400).send({ statusCode: 400, message: 'Invalid City ID' })

   await City.destroy({
    where: {id: cityId}
   }).then(() => {
     return res.status(200).send()
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

 // Return with count
const returnWithCount = async (res, filter, innerFilter, response, limit, isAdmin) => {
  if (isAdmin) {
    if (Object.keys(innerFilter).length) {
      City.count({
        where: { ...filter },
        include: [{ model: Region, as: 'region', nested: true, where: { ...innerFilter } }]
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    } else {
      City.count({
        where: { ...filter }
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    }
  } else {
    if (Object.keys(innerFilter).length) {
      City.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        }, include: [{ model: Region, as: 'region', nested: true, where: { ...innerFilter } }]
      }).then((totalCount) => {
            let result = {
              content: response,
              totalElements: totalCount,
              totalPages: Math.ceil(totalCount/limit),
            }
            return res.status(200).send(result)
        })
    } else {
      City.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        }
      }).then((totalCount) => {
            let result = {
              content: response,
              totalElements: totalCount,
              totalPages: Math.ceil(totalCount/limit),
            }
            return res.status(200).send(result)
        })
    }
  }
}
 
 module.exports = {
   add,
   getAll,
   get,
   getFiltered,
   getOne,
   update,
   remove,
   returnWithCount
 }