const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

const mailController = require('./mail.controller')

// Create Main Model

const Complaint = db.complaints
const DataOption = db.dataOptions
const UserDetail = db.userDetails
const Country = db.countries
const Region = db.regions
const City = db.cities
const Barangay = db.barangays
const User = db.users

// Create
const add = async (req, res) => {
 try {
  
  const defaultComplaintType = DataOption.findOne({ where: { code: 'OTHERS_COMPLAINTS' }, limit: 1 })

  let info = {
    id: uuidv4(),
    subject: req.body.subject,
    typeId: req.body.type ? req.body.type.brgyKey : defaultComplaintType ? defaultComplaintType.brgyKey : null,
    description: req.body.description,
    complaintById: req.body.complaintBy ? req.body.complaintBy.brgyKey : null,
    resolved: req.body.resolved ? req.body.resolved : false
   }
 
  await Complaint.create(info)
    .then((complaint) => {
      mailController.emailComplaintReceived(complaint, req.body.complaintBy.brgyKey)
      return res.status(200).send(complaint)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Create and Upload
const addAndUpload = async (req, res) => {
  try {
   const defaultComplaintType = DataOption.findOne({ where: { code: 'OTHERS_COMPLAINTS' }, limit: 1 })
   const filePath = req.body.imagePath

   let info = {
     id: uuidv4(),
     subject: req.body.subject,
     typeId: req.body.typeId ? req.body.typeId : defaultComplaintType ? defaultComplaintType.brgyKey : null,
     description: req.body.description,
     complaintById: req.body.complaintById != null && req.body.complaintById,
     attachments: filePath,
     resolved: req.body.resolved && req.body.resolved !== undefined ? req.body.resolved : false
    }
  
   await Complaint.create(info)
     .then((complaint) => {
       mailController.emailComplaintReceived(complaint, req.body.complaintById)
       return res.status(200).send(complaint)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await Complaint.findAll({
    where: { ...filter },
    include: [
      { model: DataOption, as: 'type', nested: true },
      { model: UserDetail, as: 'complaintBy', nested: true, include: [
        { model: DataOption, as: 'gender', nested: true },
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: User, as: 'user', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await Complaint.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: [
    { model: DataOption, as: 'type', nested: true },
    { model: UserDetail, as: 'complaintBy', nested: true, include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((dataOptions) => {
    return res.status(200).send(dataOptions)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let typeFilter = {}
  let userFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           if (prop === 'type') {
            typeFilter[innerprop] = req.body.filter[prop][innerprop]
           } else if ( prop === 'complaintBy') {
            userFilter[innerprop] = req.body.filter[prop][innerprop]
           }
         }
        } else {
          filter[prop] = req.body.filter[prop]
        }
      } else {
        if (prop === 'resolved') {
          filter[prop] = false
        } else if (prop === 'rejected') {
          filter[prop] = null
        }
      }
    }
  }

  await Complaint.findAll({
    where: { ...filter },
    include: [
      { model: DataOption, as: 'type', nested: true, where: { ...typeFilter } },
      { model: UserDetail, as: 'complaintBy', nested: true, where: { ...userFilter }, include: [
        { model: DataOption, as: 'gender', nested: true },
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: User, as: 'user', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let complaintId = req.params.id

  if (!complaintId) return res.status(400).send({ statusCode: 400, message: 'Invalid Complaint ID' })

  await Complaint.findOne({
   where: { id: complaintId },
   include: [
    { model: DataOption, as: 'type', nested: true },
    { model: UserDetail, as: 'complaintBy', nested: true, include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((complaint) => {
    return res.status(200).send(complaint)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let complaintId = req.params.id

  if (!complaintId) return res.status(400).send({ statusCode: 400, message: 'Invalid Complaint ID' })

  const defaultComplaintType = DataOption.findOne({ where: { code: 'OTHERS_COMPLAINTS' }, limit: 1 })

  let info = {
    subject: req.body.subject,
    typeId: req.body.type ? req.body.type.brgyKey : defaultComplaintType ? defaultComplaintType.brgyKey : null,
    description: req.body.description,
    complaintById: req.body.complaintBy ? req.body.complaintBy.brgyKey : null,
    resolved: req.body.resolved ? req.body.resolved : false
   }

  await Complaint.update(info, { where: { id: complaintId } })
  .then((complaint) => {
    return res.status(200).send(complaint)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let complaintId = req.params.id

  if (!complaintId) return res.status(400).send({ statusCode: 400, message: 'Invalid Complaint ID' })

  await Complaint.destroy({
   where: {id: complaintId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

const resolveComplaint = async (req, res) => {4
  try {
    let complaintId = req.params.id || req.body.id

    if (!complaintId) return res.status(400).send({ statusCode: 400, message: 'Invalid Complaint ID' })

    await Complaint.update({
     resolved: true,
     resolvedBy: req.body.resolvedBy,
     resolvedOn: req.body.resolvedOn
    }, { where: { id: complaintId } })
    .then((complaint) => {
      // Insert Email Completed Here
     return res.status(200).send(complaint)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

const rejectComplaint = async (req, res) => {
  try {
    let complaintId = req.params.id || req.body.id

    if (!complaintId) return res.status(400).send({ statusCode: 400, message: 'Invalid Complaint ID' })

    await Complaint.update({
     rejected: true,
     rejectionNotes: req.body.rejectionNotes,
     rejectedBy: req.body.rejectedBy
    }, { where: { id: complaintId } })
    .then((complaint) => {
     return res.status(200).send(complaint)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    Complaint.count({
      where: { ...filter }
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    Complaint.count({
      where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
    })
  }
  
}

module.exports = {
  add,
  addAndUpload,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  rejectComplaint,
  resolveComplaint,
  returnWithCount
}