const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const Country = db.countries

// Create
const add = async (req, res) => {
  try {
   let info = {
    id: uuidv4(),
    name: req.body.name,
    code: req.body.code,
    description: req.body.description,
    disabled: req.body.disabled
   }
  
   await Country.create(info)
     .then((country) => {
       return res.status(200).send(country)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All
 const getAll = async (req, res) => {
  try {
   let str = req.body && req.body.str
   let sortBy = req.body.sortBy || 'brgyKey'
   let order = req.body.order || 'ASC'
   let limit = req.body.size ? req.body.size : 20
   let pageCount = req.body.page ? req.body.page : 1
   let limitCount = 0 + (pageCount - 1) * limit
   let page = limitCount > 0 ? limitCount : 0
 
   let filter = str ? {
     [Op.or]: [
       { name: { [Op.substring] : '%' + str + '%' } },
       { code: { [Op.substring] : '%' + str + '%' } }
     ]
   } : ''
 
   await Country.findAll({
     where: {
       ...filter
     },
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All Enabled
 const get = async (req, res) => {
  try {
   await Country.findAll({
    where: {
     [Op.or]: [
       { disabled: null },
       { disabled: false },
       { deletedAt: null }
     ]
    }
   }).then((countrys) => {
     return res.status(200).send(countrys)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get Filtered
 const getFiltered = async (req, res) => {
  try {
    let sortBy = req.body.sortBy || 'brgyKey'
    let order = req.body.order || 'ASC'
    let limit = req.body.size ? req.body.size : 20
    let pageCount = req.body.page ? req.body.page : 1
    let limitCount = 0 + (pageCount - 1) * limit
    let page = limitCount > 0 ? limitCount : 0
   let filter = {}
 
   for (let prop in req.body.filter) {
     if (req.body.filter[prop]) {
       filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
     }
   }
 
   await Country.findAll({
     where: { ...filter },
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
   }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get One
 const getOne = async (req, res) => {
  try {
   let countryId = req.params.id

   if (!countryId) return res.status(400).send({ statusCode: 400, message: 'Invalid Country ID' })
 
   await Country.findOne({
    where: {id: countryId}
   }).then((country) => {
     return res.status(200).send(country)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Update One
 const update = async (req, res) => {
  try {
   let countryId = req.params.id

   if (!countryId) return res.status(400).send({ statusCode: 400, message: 'Invalid Country ID' })
   
   await Country.update(req.body, { where: { id: countryId } })
     .then((country) => {
       return res.status(200).send(country)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Delete One
 const remove = async (req, res) => {
  try {
   let countryId = req.params.id

   if (!countryId) return res.status(400).send({ statusCode: 400, message: 'Invalid Country ID' })

   await Country.destroy({
    where: {id: countryId}
   }).then(() => {
     return res.status(200).send()
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    Country.count({
      where: { ...filter }
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    Country.count({
      where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
  
}
 
 module.exports = {
   add,
   getAll,
   get,
   getFiltered,
   getOne,
   update,
   remove,
   returnWithCount
 }