const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const DataOption = db.dataOptions
const DataOptionType = db.dataOptionTypes

// Create
const add = async (req, res) => {
 try {
  let info = {
   id: uuidv4(),
   name: req.body.name,
   code: req.body.code,
   dataOptionTypeId: req.body.dataOptionType ? req.body.dataOptionType.brgyKey : null,
   description: req.body.description,
   disabled: req.body.disabled
  }
 
  await DataOption.create(info)
    .then((dataOption) => {
      return res.status(200).send(dataOption)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await DataOption.findAll({
    where: { ...filter },
    include: [{ model: DataOptionType, as: 'dataOptionType', nested: true }],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await DataOption.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: [
    {
      model: DataOptionType,
      as: 'dataOptionType'
    }
   ]
  }).then((dataOptions) => {
    return res.status(200).send(dataOptions)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let innerFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
  
  await DataOption.findAll({
    where: { ...filter },
    include: [{ model: DataOptionType, as: 'dataOptionType', nested: true, where: { ...innerFilter } }],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, innerFilter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let dataOptionId = req.params.id

  if (!dataOptionId) return res.status(400).send({ statusCode: 400, message: 'Invalid DataOption ID' })

  await DataOption.findOne({
   where: { id: dataOptionId },
   include: [{ model: DataOptionType, as: 'dataOptionType' }]
  }).then((dataOption) => {
    return res.status(200).send(dataOption)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let dataOptionId = req.params.id

  if (!dataOptionId) return res.status(400).send({ statusCode: 400, message: 'Invalid DataOption ID' })

  let info = {
    name: req.body.name,
    code: req.body.code,
    dataOptionTypeId: req.body.dataOptionType ? req.body.dataOptionType.brgyKey : null,
    description: req.body.description,
    disabled: req.body.disabled
  }

  await DataOption.update(info, { where: { id: dataOptionId } })
  .then((dataOption) => {
    return res.status(200).send(dataOption)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let dataOptionId = req.params.id

  if (!dataOptionId) return res.status(400).send({ statusCode: 400, message: 'Invalid DataOption ID' })

  await DataOption.destroy({
   where: {id: dataOptionId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

const returnWithCount = async (res, filter, innerFilter, response, limit, isAdmin) => {
  if (isAdmin) {
    if (Object.keys(innerFilter).length) {
      DataOption.count({
        where: { ...filter },
        include: [{ model: DataOptionType, as: 'dataOptionType', nested: true, where: { ...innerFilter } }]
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    } else {
      DataOption.count({
        where: { ...filter }
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    }
  } else {
    if (Object.keys(innerFilter).length) {
      DataOption.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        }, include: [{ model: DataOptionType, as: 'dataOptionType', nested: true, where: { ...innerFilter } }]
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    } else {
      DataOption.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        }
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    }
    
  }
  
}

module.exports = {
  add,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  returnWithCount
}