const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const DataOptionType = db.dataOptionTypes

// Create
const add = async (req, res) => {
 try {
  let info = {
   id: uuidv4(),
   name: req.body.name,
   code: req.body.code,
   description: req.body.description,
   disabled: req.body.disabled
  }
 
  await DataOptionType.create(info)
    .then((dataOptionType) => {
      return res.status(200).send(dataOptionType)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
       })
    })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await DataOptionType.findAll({
    where: {
      ...filter
    },
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await DataOptionType.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   }
  }).then((dataOptionTypes) => {
    return res.status(200).send(dataOptionTypes)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  
  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        filter = {
          [Op.or]: [
            { [prop]: { [Op.substring] : '%' + req.body.filter[prop] + '%' } }
          ]
        }
      }
    }
  }

  await DataOptionType.findAll({
    where: { ...filter },
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let dataOptionTypeId = req.params.id

  if (!dataOptionTypeId) return res.status(400).send({ statusCode: 400, message: 'Invalid Data Option Type ID' })

  await DataOptionType.findOne({
   where: {id: dataOptionTypeId}
  }).then((dataOptionType) => {
    return res.status(200).send(dataOptionType)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let dataOptionTypeId = req.params.id

  if (!dataOptionTypeId) return res.status(400).send({ statusCode: 400, message: 'Invalid Data Option Type ID' })

  await DataOptionType.update(req.body, { where: { id: dataOptionTypeId } })
    .then(() => {
      DataOptionType.findOne({
        where: {id: dataOptionTypeId}
       }).then((dataOptionType) => {
         return res.status(200).send(dataOptionType)
       })
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let dataOptionTypeId = req.params.id

  if (!dataOptionTypeId) return res.status(400).send({ statusCode: 400, message: 'Invalid Data Option Type ID' })

  await DataOptionType.destroy({
   where: {id: dataOptionTypeId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    DataOptionType.count({
      where: { ...filter }
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    DataOptionType.count({
      where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
  
}

module.exports = {
  add,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  returnWithCount
}