const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const mailController = require('./mail.controller')
const smsController = require('./sms.controller')

// Create Main Model

const Emergency = db.emergencies
const DataOption = db.dataOptions
const UserDetail = db.userDetails
const Country = db.countries
const Region = db.regions
const City = db.cities
const Barangay = db.barangays
const User = db.users


// Create
const add = async (req, res) => {
 try {

  let info = {
    id: uuidv4(),
    requestForAmbulance: req.body.requestForAmbulance,
    requestForFirstAid: req.body.requestForFirstAid,
    requestForFireBrigade: req.body.requestForFireBrigade,
    requestForTanodPolice: req.body.requestForTanodPolice,
    others: req.body.others,
    homeAddress: req.body.homeAddress,
    location: req.body.location,
    description: req.body.description,
    emergencyById: req.body.emergencyBy ? req.body.emergencyBy.brgyKey : null
   }
 
  await Emergency.create(info)
    .then((emergency) => {
      if (info.emergencyById) {
        UserDetail.findOne({
          where: { brgyKey: info.emergencyById },
          include: [
            { model: DataOption, as: 'gender', nested: true },
            { model: DataOption, as: 'street', nested: true },
            { model: DataOption, as: 'natureOfStay', nested: true },
            { model: User, as: 'user', nested: true },
            { model: Barangay, as: 'barangay', nested: true },
            { model: City, as: 'userCity', nested: true },
            { model: Region, as: 'userRegion', nested: true },
            { model: Country, as: 'userCountry', nested: true }
           ],
          limit: 1
         }).then((userDetails) => {
          const location = info.location ? info.location : mailController.getAddress(userDetails)
          const contactPerson = `${userDetails.firstName} ${userDetails.lastName}`
          const contactNumber = userDetails.mobile
          let message = 'Quick Emergency Response Request '
         
          if (location) {
            message += 'at ' + location + '.' 
          }
          
          if (contactPerson) {
            message += ' Requested by ' +  contactPerson + ''
          }

          if (contactNumber) {
            message += '(' + contactNumber + ').'
          } else {
            message += '.'
          }
          
          const emergencyContact = [
            {
              // Dondon
              mobile: '09193742000',
              text: message
            },
            {
              // Vener
              mobile: '09422009420',
              text: message
            },
            {
              // Austin
              mobile: '09455796157',
              text: message
            },
            {
              // Ace
              mobile: '09063330072',
              text: message
            }
          ]

          emergencyContact.forEach((sms) => {
            smsController.sendEntityMessage(sms)
          })

          mailController.emailQerrReceived(emergency, userDetails)
          return res.status(200).send(emergency)
         })
      } else {
        return res.status(200).send(emergency)
      }
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await Emergency.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'emergencyBy', nested: true, include: [
        { model: DataOption, as: 'gender', nested: true },
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: User, as: 'user', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await Emergency.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: [
    { model: UserDetail, as: 'emergencyBy', nested: true, include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((dataOptions) => {
    return res.status(200).send(dataOptions)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let userFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           if ( prop === 'emergencyBy') {
            userFilter[innerprop] = req.body.filter[prop][innerprop]
           }
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
  
  await Emergency.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'emergencyBy', nested: true, where: { ...userFilter }, include: [
        { model: DataOption, as: 'gender', nested: true },
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: User, as: 'user', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
       ]}
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let emergencyId = req.params.id

  if (!emergencyId) return res.status(400).send({ statusCode: 400, message: 'Invalid Emergency ID' })

  await Emergency.findOne({
   where: { id: emergencyId },
   include: [
    { model: UserDetail, as: 'emergencyBy', nested: true, include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]}
   ]
  }).then((emergency) => {
    return res.status(200).send(emergency)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let emergencyId = req.params.id

  if (!emergencyId) return res.status(400).send({ statusCode: 400, message: 'Invalid Emergency ID' })

  let info = {
    id: uuidv4(),
    requestForAmbulance: req.body.requestForAmbulance,
    requestForFirstAid: req.body.requestForFirstAid,
    requestForFireBrigade: req.body.requestForFireBrigade,
    requestForTanodPolice: req.body.requestForTanodPolice,
    others: req.body.others,
    homeAddress: req.body.homeAddress,
    location: req.body.location,
    description: req.body.description,
    emergencyById: req.body.emergencyBy ? req.body.emergencyBy.brgyKey : null,
    resolved: req.body.resolved ? req.body.resolved : false
  }

  await Emergency.update(info, { where: { id: emergencyId } })
  .then((emergency) => {
    return res.status(200).send(emergency)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let emergencyId = req.params.id

  if (!emergencyId) return res.status(400).send({ statusCode: 400, message: 'Invalid Emergency ID' })

  await Emergency.destroy({
   where: {id: emergencyId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

const resolveEmergency = async (req, res) => {4
  try {
    let emergencyId = req.params.id || req.body.id

    if (!emergencyId) return res.status(400).send({ statusCode: 400, message: 'Invalid Emergency ID' })

    await Emergency.update({
     resolved: true,
     resolvedBy: req.body.resolvedBy,
     resolvedOn: req.body.resolvedOn
    }, { where: { id: emergencyId } })
    .then((emergency) => {
     return res.status(200).send(emergency)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

const rejectEmergency = async (req, res) => {
  try {
    let emergencyId = req.params.id || req.body.id

    if (!emergencyId) return res.status(400).send({ statusCode: 400, message: 'Invalid Emergency ID' })

    await Emergency.update({
     rejected: true,
     rejectionNotes: req.body.rejectionNotes,
     rejectedBy: req.body.rejectedBy
    }, { where: { id: emergencyId } })
    .then((emergency) => {
     return res.status(200).send(emergency)
    }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    })
    
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
}

const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    Emergency.count({
      where: { ...filter }
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    Emergency.count({
      where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
  
}

module.exports = {
  add,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  rejectEmergency,
  resolveEmergency,
  returnWithCount
}