const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const News = db.news
const UserDetail = db.userDetails

// Create
const add = async (req, res) => {
 try {
  let info = {
    id: uuidv4(),
    title: req.body.title,
    description: req.body.description,
    disabled: req.body.disabled,
    createdById: req.body.createdById,
    tags: req.body.tags != null && req.body.tags.length > 0 ? req.body.tags.join() : ''
  }
 
  await News.create(info)
    .then((news) => {
      return res.status(200).send(news)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Create and Upload
const addAndUpload = async (req, res) => {
  try {
   const filePath = req.body.imagePath

   let info = {
     id: uuidv4(),
     title: req.body.title,
     description: req.body.description,
     disabled: req.body.disabled,
     createdById: req.body.createdById,
     attachments: filePath,
     tags: req.body.tags
    }
  
   await News.create(info)
     .then((news) => {
       return res.status(200).send(news)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

// Update and Upload
const addAndUploadUpdate = async (req, res) => {
  try {
   const filePath = req.body.imagePath

   let info = {
      title: req.body.title,
      description: req.body.description,
      disabled: req.body.disabled,
      createdById: req.body.createdById,
      attachments: filePath,
      tags: req.body.tags
    }
  
   await News.update(info, { where: { id: req.body.id } })
     .then(() => {
      News.findOne({ where: { id: req.body.id } })
      .then((news) => {
        if (news.tags) {
          news.tags = news.tags.split(',')
        }
        return res.status(200).send(news)
      })
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

// Get All (Admin)
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? { title: { [Op.substring] : '%' + str + '%' } } : ''

  await News.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'createdBy', nested: true }
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await News.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: [
    { model: UserDetail, as: 'createdBy', nested: true }
   ]
  }).then((news) => {
    return res.status(200).send(news)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let innerFilter = {}
 
   if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
  
  await News.findAll({
    where: { ...filter },
    include: [
      { model: UserDetail, as: 'createdBy', nested: true, where: { ...innerFilter } }
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, innerFilter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Client)
const getFilteredClient = async (req, res) => {
  try {
   let sortBy = req.body.sortBy || 'brgyKey'
   let order = req.body.order || 'ASC'
   let limit = req.body.size ? req.body.size : 20
   let pageCount = req.body.page ? req.body.page : 1
   let limitCount = 0 + (pageCount - 1) * limit
   let page = limitCount > 0 ? limitCount : 0
   let filter = {}
   let innerFilter = {}
  
    if (req.body.filter) {
     for (let prop in req.body.filter) {
       if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
     }
   }
   
   await News.findAll({
     where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { deletedAt: null }
        ]
      },
     include: [
       { model: UserDetail, as: 'createdBy', nested: true, where: { ...innerFilter } }
     ],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
     return returnWithCount(res, filter, innerFilter, response, limit, false)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

// Get One
const getOne = async (req, res) => {
 try {
  let newsId = req.params.id

  if (!newsId) return res.status(400).send({ statusCode: 400, message: 'Invalid News ID' })

  await News.findOne({
   where: { id: newsId },
   include: [
    { model: UserDetail, as: 'createdBy', nested: true }
   ]
  }).then((news) => {
    if (news.tags) {
      news.tags = news.tags.split(',')
    }
    return res.status(200).send(news)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let newsId = req.params.id

  if (!newsId) return res.status(400).send({ statusCode: 400, message: 'Invalid News ID' })

  let info = {
    title: req.body.title,
    description: req.body.description,
    disabled: req.body.disabled,
    createdById: req.body.createdById,
    tags: req.body.tags != null && req.body.tags.length > 0 ? req.body.tags.join() : ''
   }

  await News.update(info, { where: { id: newsId } })
  .then((news) => {
    return res.status(200).send(news)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let newsId = req.params.id

  if (!newsId) return res.status(400).send({ statusCode: 400, message: 'Invalid News ID' })

  await News.destroy({
   where: {id: newsId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Return with count
const returnWithCount = async (res, filter, innerFilter, response, limit, isAdmin) => {
  if (isAdmin) {
    if (Object.keys(innerFilter).length) {
      News.count({
        where: { ...filter },
        include: [
          { model: UserDetail, as: 'createdBy', nested: true, where: { ...innerFilter } }
        ]
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    } else {
      News.count({
        where: { ...filter }
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    }
    
  } else {
    if (Object.keys(innerFilter).length) {
      News.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        },
        include: [
          { model: UserDetail, as: 'createdBy', nested: true, where: { ...innerFilter } }
        ]
      }).then((totalCount) => {
        let result = {
          content: response,
          totalElements: totalCount,
          totalPages: Math.ceil(totalCount/limit),
        }
        return res.status(200).send(result)
      })
    } else {
      News.count({
        where: { ...filter,
          [Op.or]: [
            { disabled: null },
            { disabled: false }
          ],
          [Op.and]: [
            { deletedAt: null }
          ]
        }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
    }
  }
  
}

module.exports = {
  add,
  addAndUpload,
  addAndUploadUpdate,
  getAll,
  get,
  getFiltered,
  getFilteredClient,
  getOne,
  update,
  remove,
  returnWithCount
}