const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const Region = db.regions
const Country = db.countries

// Create
const add = async (req, res) => {
  try {
   let info = {
    id: uuidv4(),
    name: req.body.name,
    code: req.body.code,
    description: req.body.description,
    countryId: req.body.country ? req.body.country.brgyKey : null,
    disabled: req.body.disabled
   }
  
   await Region.create(info)
     .then((region) => {
       return res.status(200).send(region)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All
 const getAll = async (req, res) => {
  try {
   let str = req.body && req.body.str
   let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
 
   let filter = str ? {
     [Op.or]: [
       { name: { [Op.substring] : '%' + str + '%' } },
       { code: { [Op.substring] : '%' + str + '%' } }
     ]
   } : ''
 
   await Region.findAll({
     where: { ...filter },
     include: [{ model: Country, as: 'country', nested: true }],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All Enabled
 const get = async (req, res) => {
  try {
   await Region.findAll({
    where: {
     [Op.or]: [
       { disabled: null },
       { disabled: false },
       { deletedAt: null }
     ]
    },
    include: [
     {
       model: Country,
       as: 'country'
     }
    ]
   }).then((regions) => {
     return res.status(200).send(regions)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get Filtered (Admin)
 const getFiltered = async (req, res) => {
  try {
    let sortBy = req.body.sortBy || 'brgyKey'
    let order = req.body.order || 'ASC'
    let limit = req.body.size ? req.body.size : 20
    let pageCount = req.body.page ? req.body.page : 1
    let limitCount = 0 + (pageCount - 1) * limit
    let page = limitCount > 0 ? limitCount : 0
   let filter = {}
   let innerFilter = {}
 
   if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
   
   await Region.findAll({
     where: { ...filter },
     include: [{ model: Country, as: 'country', nested: true, where: { ...innerFilter } }],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, innerFilter, response, limit)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get One
 const getOne = async (req, res) => {
  try {
   let regionId = req.params.id

   if (!regionId) return res.status(400).send({ statusCode: 400, message: 'Invalid Region ID' })
 
   await Region.findOne({
    where: { id: regionId },
    include: [{ model: Country, as: 'country' }]
   }).then((region) => {
     return res.status(200).send(region)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Update One
 const update = async (req, res) => {
  try {
   let regionId = req.params.id

   if (!regionId) return res.status(400).send({ statusCode: 400, message: 'Invalid Region ID' })

   let info = {
    name: req.body.name,
    code: req.body.code,
    description: req.body.description,
    countryId: req.body.country ? req.body.country.brgyKey : null,
    disabled: req.body.disabled
   }
 
   await Region.update(info, { where: { id: regionId } })
   .then((region) => {
     return res.status(200).send(region)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Delete One
 const remove = async (req, res) => {
  try {
   let regionId = req.params.id

   if (!regionId) return res.status(400).send({ statusCode: 400, message: 'Invalid Region ID' })

   await Region.destroy({
    where: {id: regionId}
   }).then(() => {
     return res.status(200).send()
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

 // Return with count
const returnWithCount = async (res, filter, innerFilter, response, limit) => {
  if (Object.keys(innerFilter).length) {
    Region.count({
      where: { ...filter }, include: [{ model: Country, as: 'country', nested: true, where: { ...innerFilter } }]
      }).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  } else {
    Region.count({
      where: { ...filter }
      }).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
}
 
 module.exports = {
   add,
   getAll,
   get,
   getFiltered,
   getOne,
   update,
   remove,
   returnWithCount
 }