const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const config = require('../config')[process.env.NODE_ENV || 'development']
const mailController = require('./mail.controller')
const log = config.log()

const User = db.users
const Registration = db.registration

// Email Validation
const validateEmail = async (req, res) => {
    try {
        const { email } = req.body
        const users = await User.findAll({
            where: { email },
            limit: 1
        })

        if (users.length > 0) {
            // Email already exists in the database
            return res.status(200).json({ exists: true, message: 'Email already exists' });
        } else {
            // Email does not exist in the database
            return res.status(200).json({ exists: false, message: 'Email does not exist' });
        }
    } catch (error) {
        // Handle any errors that occur during the process
        return res.status(500).json({ error: 'An error occurred while checking the email' });
    }
}

// Request OTP Authentication
const requestAuthentication = async (req, res) => {
    try {
        let code = await generateOTP()
        let info = {
            id: uuidv4(),
            email: req.body.email,
            firstName: req.body.firstName,
            lastName: req.body.lastName,
            code: code,
            status: 'PENDING'
        }

        await Registration.create(info)
            .then((response) => {
                mailController.sendOTPByEmail(info)
                const { code, ...newResponse } = response.dataValues
                return res.status(200).send({
                    statusCode: 200,
                    message: 'OTP Sent Successfully',
                    content: newResponse
                })
            }).catch((err) => {
                console.error(err)
                res.status(500).json({ error: 'Failed to send OTP.' })
            })
    } catch (error) {
        res.status(500).json({ error: 'Failed to send OTP.' })
    }
}

// Resend OTP Authentication
const resendAuthentication = async (req, res) => {
    try {
        if (!req.body.id) res.status(500).json({ error: 'Invalid Request!' })

        let info = {}
        for (let prop in req.body) {
            info[prop] = req.body[prop]
        }
        info.code = generateOTP()

        await Registration.update(info, {
            where: { id: info.id }
        }).then(() => {
            mailController.sendOTPByEmail(info)
            const { code, ...newResponse } = info
            return res.status(200).send({
                statusCode: 200,
                message: 'OTP Sent Successfully',
                content: newResponse
            })
        }).catch((err) => {
            console.error(err)
            res.status(500).json({ error: 'Failed to send OTP.' })
        })
    } catch (error) {
        res.status(500).json({ error: 'Failed to send OTP.' })
    }
}

const generateOTP = async () => {
    let code = Math.floor(100000 + Math.random() * 900000)
    return `${code}`
}

const validateAuthentication = async (req, res) => {
    try {
        if (!req.body.id) return res.status(500).json({ error: 'Invalid Request!' })

        let existingRequest = await Registration.findOne({ where: { id: req.body.id } })

        if (!existingRequest) return res.status(500).json({ error: 'Invalid Request!' })

        let info = {}
        for (let prop in req.body) {
            info[prop] = req.body[prop]
        }

        if (existingRequest.code === info.code) {
            return res.status(200).send({
                statusCode: 200,
                message: 'Success',
                matched: true
            })
        } else {
            return res.status(200).send({
                statusCode: 400,
                message: 'Invalid OTP',
                matched: false
            })
        }
    } catch (error) {
        res.status(500).json({ error: 'Failed to send OTP.' })
    }
}

// Registration (Step 1)
const register = async (req, res) => {
    // If not, generate URL for Email Verification and Send Email Verification Email
    // Else, reject the request and show validation in the UI
}

// Registration Email Verification (Step 2)
const emailVerification = async (req, res) => {
    // Check if the token is still valid,
    // If not, then mark the registration as deleted
    // Else, proceed on providing the ID of the user to UI
}

// Registration User Details (Step 3)
const userDetails = async (req, res) => {
    // Handles all the new fields for the User Details
    // Change Status
}

const uploadVerification = async (req, res) => {
    // Handles verification image upload
}

module.exports = {
    validateEmail,
    requestAuthentication,
    resendAuthentication,
    validateAuthentication,
    register,
    emailVerification,
    userDetails,
    uploadVerification
}