const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const moment = require('moment')

const DocumentService = require('../services/document.service.js')
const mailController = require('./mail.controller')
const smsController = require('./sms.controller')

const RequestDocument = db.requestDocument
const UserDetails = db.userDetails
const User = db.users
const DataOption = db.dataOptions
const Country = db.countries
const Region = db.regions
const City = db.cities
const Barangay = db.barangays

// Request Document - Certificate with Cedula Photo
const createRequestDocumentWithCedula = async (req, res) => {
  try {
    let documentTypeCode = req.body.documentTypeCode
    let ticketNumber = await generateTicketNumber()
    let userPath = req.body.userPath
    let cedulaPath = req.body.cedulaPath

    if (documentTypeCode && documentTypeCode === 'CERTIFICATE' && userPath && cedulaPath && ticketNumber) {

      let request = {
        id: uuidv4(),
        ticketNumber: ticketNumber,
        documentCategoryId: req.body.documentCategoryId,
        requestedById: req.body.requestedById,
        yearsOfStay: req.body.yearsOfStay,
        monthsOfStay: req.body.monthsOfStay,
        purposeId: req.body.purposeId,
        purposeOthers: req.body.purposeOthers || null,
        userPhoto: userPath,
        cedulaPhoto: cedulaPath,
        remarks: req.body.remarks,
        statusId: req.body.statusId,
        requestedOn: new Date()
      }

      log.info('Start saving request document...')

      await RequestDocument.create(request)
       .then((data) => {
          RequestDocument.findOne({
            where: { id: data.dataValues.id },
            include: [
              { model: DataOption, as: 'requestStatus', nested: true },
              { model: DataOption, as: 'purpose', nested: true },
              { model: DataOption, as: 'documentCategory', nested: true },
              { model: DataOption, as: 'civilStatus', nested: true },
              { model: UserDetails, as: 'requestedBy', nested: true,
                include: [
                  { model: DataOption, as: 'gender', nested: true },
                  { model: DataOption, as: 'street', nested: true },
                  { model: DataOption, as: 'natureOfStay', nested: true },
                  { model: User, as: 'user', nested: true },
                  { model: Barangay, as: 'barangay', nested: true },
                  { model: City, as: 'userCity', nested: true },
                  { model: Region, as: 'userRegion', nested: true },
                  { model: Country, as: 'userCountry', nested: true }
                ]
              }
            ]
          }).then((response) => {
            mailController.sendRequestDocumentInitialEmail(response)
          })

          return res.status(200).send(data.dataValues)
        }).catch((error) => {
          log.fatal('Could not save request document (createRequestDocumentWithCedula):', error)
          return res.status(400).send({
            statusCode: 400,
            message: error
          })
        })
    } else {
      return res.status(400).send({
        statusCode: 400,
        message: 'Invalid Data (createRequestDocumentWithCedula)'
      })
    }
  } catch (error) {
    log.fatal('Error creating request document (createRequestDocumentWithCedula):', error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Request Document - without Cedula Photo (2 Request)
const createRequestDocumentWithoutCedula = async (req, res) => {
  try {
    let documentTypeCode = req.body.documentTypeCode
    let ticketNumber = await generateTicketNumber()
    let getCedula = await DataOption.findOne({ where: { code: 'CEDULA' } })
    let cedulaDefault = getCedula && getCedula.dataValues
    let userPath = req.body.userPath

    if (documentTypeCode && documentTypeCode === 'CERTIFICATE_CEDULA' && userPath && ticketNumber && cedulaDefault) {

      let certificate = {
        id: uuidv4(),
        ticketNumber: ticketNumber,
        documentCategoryId: req.body.documentCategoryId,
        requestedById: req.body.requestedById,
        yearsOfStay: req.body.yearsOfStay,
        monthsOfStay: req.body.monthsOfStay,
        purposeId: req.body.purposeId,
        purposeOthers: req.body.purposeOthers || null,
        userPhoto: userPath,
        remarks: req.body.remarks,
        statusId: req.body.statusId,
        requestedOn: new Date()
      }

      let cedula = {
        id: uuidv4(),
        ticketNumber: ticketNumber,
        documentCategoryId: cedulaDefault && cedulaDefault.brgyKey,
        requestedById: req.body.requestedById,
        civilStatusId: req.body.civilStatusId,
        annualIncome: req.body.annualIncome,
        citizenship: req.body.citizenship,
        statusId: req.body.statusId,
        requestedOn: new Date()
      }

      const saveCertificate = DocumentService.saveCertificate(certificate)
      const saveCedula = DocumentService.saveCedula(cedula)

      log.info('Start saving request document...')

      await Promise.all([saveCertificate, saveCedula])
        .then((data) => {
          data.forEach(async (item) => {
            await RequestDocument.findOne({
              where: { id: item.id },
              include: [
                { model: DataOption, as: 'requestStatus', nested: true },
                { model: DataOption, as: 'purpose', nested: true },
                { model: DataOption, as: 'documentCategory', nested: true },
                { model: DataOption, as: 'civilStatus', nested: true },
                { model: UserDetails, as: 'requestedBy', nested: true,
                  include: [
                    { model: DataOption, as: 'gender', nested: true },
                    { model: DataOption, as: 'street', nested: true },
                    { model: DataOption, as: 'natureOfStay', nested: true },
                    { model: User, as: 'user', nested: true },
                    { model: Barangay, as: 'barangay', nested: true },
                    { model: City, as: 'userCity', nested: true },
                    { model: Region, as: 'userRegion', nested: true },
                    { model: Country, as: 'userCountry', nested: true }
                  ]
                }
              ]
            }).then((response) => {
              mailController.sendRequestDocumentInitialEmail(response)
            })
          })
          
          log.info('Request document saved with ticket number:', ticketNumber)
          return res.status(200).send(data)
        }).catch((error) => {
          log.fatal('Could not save request document (createRequestDocumentWithoutCedula):', error)
          return res.status(400).send({
            statusCode: 400,
            message: error
          })
        })
    } else {
      return res.status(400).send({
        statusCode: 400,
        message: 'Invalid Data (createRequestDocumentWithoutCedula)'
      })
    }
  } catch (error) {
    log.fatal('Error creating request document (createRequestDocumentWithoutCedula):', error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Request Document - Cedula (Working)
const createRequestDocumentCedula = async (req, res) => {
  try {
    let documentTypeCode = req.body.documentTypeCode
    let ticketNumber = await generateTicketNumber()

    if (documentTypeCode && documentTypeCode === 'CEDULA' && ticketNumber) {

      let cedula = {
        id: uuidv4(),
        ticketNumber: ticketNumber,
        documentCategoryId: req.body.documentCategory ? req.body.documentCategory.brgyKey : null,
        requestedById: req.body.requestedBy ? req.body.requestedBy.brgyKey : null,
        civilStatusId: req.body.civilStatus ? req.body.civilStatus.brgyKey : null,
        statusId: req.body.status ? req.body.status.brgyKey : null,
        annualIncome: req.body.annualIncome,
        citizenship: req.body.citizenship,
        requestedOn: new Date()
      }

      log.info('Start saving request document...')

      await RequestDocument.create(cedula)
       .then((data) => {
          RequestDocument.findOne({
            where: { id: data.dataValues.id },
            include: [
              { model: DataOption, as: 'requestStatus', nested: true },
              { model: DataOption, as: 'purpose', nested: true },
              { model: DataOption, as: 'documentCategory', nested: true },
              { model: DataOption, as: 'civilStatus', nested: true },
              { model: UserDetails, as: 'requestedBy', nested: true,
                include: [
                  { model: DataOption, as: 'gender', nested: true },
                  { model: DataOption, as: 'street', nested: true },
                  { model: DataOption, as: 'natureOfStay', nested: true },
                  { model: User, as: 'user', nested: true },
                  { model: Barangay, as: 'barangay', nested: true },
                  { model: City, as: 'userCity', nested: true },
                  { model: Region, as: 'userRegion', nested: true },
                  { model: Country, as: 'userCountry', nested: true }
                ]
              }
            ]
          }).then((response) => {
            mailController.sendRequestDocumentInitialEmail(response)
          })

          log.info('Request document saved with ID:', data.id)
          return res.status(200).send(data.dataValues)
        }).catch((error) => {
          log.fatal('Could not save request document (createRequestDocumentCedula):', error)
          return res.status(400).send({
            statusCode: 400,
            message: error
          })
        })
    } else {
      return res.status(400).send({
        statusCode: 400,
        message: 'Invalid Data (createRequestDocumentCedula)'
      })
    }
  } catch (error) {
    log.fatal('Error creating request document (createRequestDocumentCedula):', error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Generate Ticket Number
const generateTicketNumber = () => {
  return new Promise((resolve, reject) => {
    let dateFormat = moment(new Date()).tz('Asia/Manila').format('YYMMDD')
    RequestDocument.count()
      .then((totalCount) => {
        let outputCount = String(totalCount).padStart(4, '0')
        const tn = `${dateFormat}${outputCount}`
        log.info('Generating Request Document Ticket Number', tn)
        resolve(tn)
      }).catch(reject)
  })
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? {
    [Op.or]: [
      { name: { [Op.substring] : '%' + str + '%' } },
      { code: { [Op.substring] : '%' + str + '%' } }
    ]
  } : ''

  await RequestDocument.findAll({
    where: { ...filter },
    include: [{ model: UserDetails, as: 'requestedBy', nested: true }],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, {}, response, limit)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
  try {
    await RequestDocument.findAll({
      where: { [Op.and]: [ { deletedAt: null } ] },
      include: [
        { model: DataOption, as: 'requestStatus', nested: true },
        { model: DataOption, as: 'purpose', nested: true },
        { model: DataOption, as: 'documentCategory', nested: true },
        { model: DataOption, as: 'civilStatus', nested: true },
        { model: UserDetails, as: 'requestedBy', nested: true,
          include: [
            { model: DataOption, as: 'gender', nested: true },
            { model: DataOption, as: 'street', nested: true },
            { model: DataOption, as: 'natureOfStay', nested: true },
            { model: User, as: 'user', nested: true },
            { model: Barangay, as: 'barangay', nested: true },
            { model: City, as: 'userCity', nested: true },
            { model: Region, as: 'userRegion', nested: true },
            { model: Country, as: 'userCountry', nested: true }
          ]
        }
      ]
    }).then((response) => {
      return res.status(200).send(response)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
  let innerFilter = {}

  if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
  
  await RequestDocument.findAll({
    where: { ...filter },
    include: [
      { model: DataOption, as: 'requestStatus', nested: true },
      { model: DataOption, as: 'purpose', nested: true },
      { model: DataOption, as: 'documentCategory', nested: true },
      { model: DataOption, as: 'civilStatus', nested: true },
      { model: UserDetails, as: 'requestedBy', nested: true, where: { ...innerFilter },
        include: [
          { model: DataOption, as: 'gender', nested: true },
          { model: DataOption, as: 'street', nested: true },
          { model: DataOption, as: 'natureOfStay', nested: true },
          { model: User, as: 'user', nested: true },
          { model: Barangay, as: 'barangay', nested: true },
          { model: City, as: 'userCity', nested: true },
          { model: Region, as: 'userRegion', nested: true },
          { model: Country, as: 'userCountry', nested: true }
        ]
      }
    ],
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
     return returnWithCount(res, filter, innerFilter, response, limit)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOne = async (req, res) => {
 try {
  let requestDocumentId = req.params.id

  if (!requestDocumentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Document ID' })

  await RequestDocument.findOne({
    where: { id: requestDocumentId },
    include: [
      { model: DataOption, as: 'requestStatus', nested: true },
      { model: DataOption, as: 'purpose', nested: true },
      { model: DataOption, as: 'documentCategory', nested: true },
      { model: DataOption, as: 'civilStatus', nested: true },
      { model: UserDetails, as: 'requestedBy', nested: true,
        include: [
          { model: DataOption, as: 'gender', nested: true },
          { model: DataOption, as: 'street', nested: true },
          { model: DataOption, as: 'natureOfStay', nested: true },
          { model: User, as: 'user', nested: true },
          { model: Barangay, as: 'barangay', nested: true },
          { model: City, as: 'userCity', nested: true },
          { model: Region, as: 'userRegion', nested: true },
          { model: Country, as: 'userCountry', nested: true }
        ]
      }
    ]
  }).then((response) => {
    return res.status(200).send(response)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let requestDocumentId = req.params.id
  let info = {}

  if (!requestDocumentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Document ID' })

  if (req.body.status) {
    info = {
      statusId: req.body.status && req.body.status.brgyKey
    }
  } else if (req.body.statusCode) {
    const selectedStatus = await DataOption.findOne({ where: { code: req.body.statusCode }, limit: 1 })
    info = {
      statusId: selectedStatus && selectedStatus.brgyKey
    }
  }

  await RequestDocument.update(info, { where: { id: requestDocumentId } })
  .then((response) => {
      RequestDocument.findOne({
        where: { id: requestDocumentId },
        include: [
          { model: DataOption, as: 'requestStatus', nested: true },
          { model: DataOption, as: 'purpose', nested: true },
          { model: DataOption, as: 'documentCategory', nested: true },
          { model: DataOption, as: 'civilStatus', nested: true },
          { model: UserDetails, as: 'requestedBy', nested: true,
            include: [
              { model: DataOption, as: 'gender', nested: true },
              { model: DataOption, as: 'street', nested: true },
              { model: DataOption, as: 'natureOfStay', nested: true },
              { model: User, as: 'user', nested: true },
              { model: Barangay, as: 'barangay', nested: true },
              { model: City, as: 'userCity', nested: true },
              { model: Region, as: 'userRegion', nested: true },
              { model: Country, as: 'userCountry', nested: true }
            ]
          }
        ]
      }).then((data) => {
        console.log('REQUEST DOCS DATA', data)
          let converted = JSON.stringify(data)
          let entity = JSON.parse(converted)
          let statusCode = entity && entity.requestStatus && entity.requestStatus.code

          if (statusCode === 'R_UNPAID' || statusCode === 'R_PAID') {
            mailController.sendRequestDocumentReleaseEmail(entity)
            if (entity.requestedBy && entity.requestedBy.mobile) {
              const sms = {
                mobile: entity.requestedBy.mobile,
                text: 'Hello, kabarangay! Maaari mo nang ipick-up ang iyong Barangay Certification. Magtungo lang sa Bulwagang Maharlika, 6AM-8PM. Salamat! #TatakIsangNamayan'
              }
              smsController.sendEntityMessage(sms)
            }
            return res.status(200).send(entity)
          } else if (statusCode === 'RD_CANCELLED') {
            mailController.sendRequestDocumentCancelledEmail(entity)
            return res.status(200).send(entity)
          } else {
            return res.status(200).send(entity)
          }
       })
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
  try {
    let requestDocumentId = req.params.id
    if (!requestDocumentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Request Document ID' })
    await RequestDocument.destroy({
      where: {id: requestDocumentId}
    }).then(() => {
      return res.status(200).send()
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const returnWithCount = async (res, filter, innerFilter, response, limit) => {
 if (Object.keys(innerFilter).length) {
  await RequestDocument.count({
   where: { ...filter },
   include: [{ model: UserDetails, as: 'requestedBy', nested: true, where: { ...innerFilter } }],
  }).then((totalCount) => {
     let result = {
       content: response,
       totalElements: totalCount,
       totalPages: Math.ceil(totalCount/limit),
     }
     return res.status(200).send(result)
  })
 } else {
  await RequestDocument.count({
   where: { ...filter }
  }).then((totalCount) => {
     let result = {
       content: response,
       totalElements: totalCount,
       totalPages: Math.ceil(totalCount/limit),
     }
     return res.status(200).send(result)
  })
 }
 
}

module.exports = {
  createRequestDocumentWithCedula,
  createRequestDocumentWithoutCedula,
  createRequestDocumentCedula,
  generateTicketNumber,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  returnWithCount
}