const fs = require('fs')
const path = require('path')
const multer = require('multer')
const sharp = require('sharp')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const mb = 1000000

// Function to define the path for saving images based on the entity type
function getImageSavePath(entityType) {
    // Get current date in yyyy-mm-dd format
    const currentDate = new Date().toISOString().slice(0, 10)
    // Define the base path for saving images
    let basePath = ''
    switch (entityType.toUpperCase()) {
        case 'BANNER':
            basePath = path.join(config.filePath, 'banners', currentDate)
            break
        case 'NEWS':
            basePath = path.join(config.filePath, 'news', currentDate)
            break
        case 'ANNOUNCEMENT':
            basePath = path.join(config.filePath, 'announcements', currentDate)
            break
        case 'COMPLAINT':
            basePath = path.join(config.filePath, 'complaints', currentDate)
            break
        case 'DOCUMENT':
            basePath = path.join(config.filePath, 'request-documents', currentDate)
            break
        case 'OTHER_ENTITY_TYPE':
            basePath = path.join(config.filePath, 'other_entity', currentDate)
            break
        // Add more cases as needed for other entity types
        default:
            log.error('Invalid entity type:', entityType)
            return null
    }
    return basePath
}

// Define Multer storage for entities
function getMulterStorage(entityType, useMemoryStorage = false) {
    if (useMemoryStorage) {
        return multer.memoryStorage();
    }

    return multer.diskStorage({
        destination: function (req, file, cb) {
            // Use the same path as defined in Multer storage
            const savePath = getImageSavePath(entityType)
            if (!savePath) {
                cb('Invalid entity type', null)
            } else {
                // Check if the destination folder exists, if not, create it
                try {
                    fs.mkdirSync(savePath, { recursive: true })
                } catch (err) {
                    // Handle any errors if directory creation fails
                    cb(err, null)
                    return // Exit the function to avoid executing the next step
                }

                // Use the same path whether the directory exists or is newly created
                cb(null, savePath)
            }
        },
        filename: function (req, file, cb) {
            if (file || file !== undefined) {
                const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
                cb(null, file.fieldname + '-' + uniqueSuffix + '-' + file.originalname)
            } else {
                log.fatal('Error uploading image: No Attachment Found...')
                cb('ERROR: NO ATTACHMENT FOUND...', false)
            }
        },
        fileFilter: (req, file, cb) => {
            let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
            cb(null, valid)
        }
    })
}

function getRelativePath(entityType) {
    switch (entityType.toUpperCase()) {
        case 'BANNER':
            return 'banners'
        case 'NEWS':
            return 'news'
        case 'ANNOUNCEMENT':
            return 'announcements'
        case 'COMPLAINT':
            return 'complaints'
        case 'DOCUMENT':
            return 'request-document'
        case 'OTHER_ENTITY_TYPE':
            return 'other_entity'
        default:
            log.error('Invalid entity type:', entityType)
            return null
    }
}

// Function to process file optimization
async function processFileOptimization(req, entityType) {

    console.log('processFileOptimization', req)
    // Check if file is present
    if (!req.file) {
        log.fatal('No file uploaded!')
        return next()
    }

    // Use Sharp to optimize and resize the image
    const savePath = getImageSavePath(entityType)
    if (!savePath) {
        log.error('Invalid entity type:', entityType)
        return null
    }
    const optimizedPath = path.join(savePath, req.file.originalname) // Use original name for saving
    const relativePath = path.join('images', getRelativePath(entityType), new Date().toISOString().slice(0, 10), req.file.originalname)

    try {
        // Resize the image using the resizeImage function
        const resizedBuffer = await resizeImage(req.file.buffer, entityType)

        // Ensure the directory exists
        await fs.promises.mkdir(savePath, { recursive: true })

        // Write the resized image to file
        await sharp(resizedBuffer)
            .jpeg({ quality: 80 }) // Adjust as needed
            .toFile(optimizedPath)

        return relativePath
    } catch (err) {
        log.error('Error processing image:', err)
        return null
    }

    // try {
    //     // Resize the image using the resizeImage function
    //     const resizedBuffer = await resizeImage(req.file.buffer, entityType)

    //     // Write the resized image to file
    //     await sharp(resizedBuffer)
    //         .jpeg({ quality: 80 }) // Set JPEG quality (optional)
    //         .toFile(optimizedPath)

    //     return optimizedPath
    // } catch (err) {
    //     log.error('Error processing image:', err)
    //     return null
    // }
}

function uploadBanner() {
    const entityType = 'BANNER';
    const upload = multer({
        storage: getMulterStorage('BANNER', true),
        limits: { fileSize: 50 * mb }
    }).single('file');

    return function (req, res, next) {
        // Handle file upload
        upload(req, res, async (err) => {
            if (err) {
                return next(err); // Pass error to the next middleware
            }

            try {
                // Process file optimization
                const optimizedPath = await processFileOptimization(req, entityType);
                req.optimizedImagePath = optimizedPath; // Store the optimized image path in the request object for further processing if needed
                next();
            } catch (error) {
                // Handle any errors
                console.error(`Error uploading and optimizing ${entityType}:`, error);
                next(error); // Pass error to the next middleware
            }
        });
    }
}

// function uploadNews() {
//     const upload = multer({
//         storage: getMulterStorage('NEWS'),
//         limits: { fileSize: 50 * mb }
//     }).single('file')

//     return uploadAndOptimize(upload, req, res, next)
// }
// function uploadAnnouncements() {
//     const upload = multer({
//         storage: getMulterStorage('ANNOUNCEMENT'),
//         limits: { fileSize: 50 * mb }
//     }).single('file')

//     return uploadAndOptimize(upload, req, res, next)
// }
// function uploadComplaints() {
//     const upload = multer({
//         storage: getMulterStorage('COMPLAINT'),
//         limits: { fileSize: 50 * mb }
//     }).single('file')

//     return uploadAndOptimize(upload, req, res, next)
// }
// function uploadDococument() {
//     const upload = multer({
//         storage: getMulterStorage('DOCUMENT'),
//         limits: { fileSize: 50 * mb }
//     }).single('file')

//     return uploadAndOptimize(upload, req, res, next)
// }

// Middleware function for uploading and optimizing images for different entities
function uploadAndOptimize(upload) {
    return function (req, res, next) {
        // Handle file upload
        upload(req, res, async (err) => {
            if (err) {
                return next(err) // Pass error to the next middleware
            }

            try {
                // Process file optimization
                const optimizedPath = await processFileOptimization(req, entityType)
                req.optimizedImagePath = optimizedPath // Store the optimized image path in the request object for further processing if needed
                next()
            } catch (error) {
                // Handle any errors
                console.error(`Error uploading and optimizing ${entityType}:`, error)
                next(error) // Pass error to the next middleware
            }
        })
    }
}

// Function to resize images based on entity type
async function resizeImage(buffer, entityType) {
    let width = 800 // Default width
    // Customize width based on entity type if needed
    switch (entityType.toUpperCase()) {
        case 'BANNER':
            // Example: Set specific width for banners
            width = 1200
            break
        case 'OTHER_ENTITY_TYPE':
            // Example: Set specific width for other entity types
            width = 1000
            break
        // Add more cases as needed for other entity types
        default:
            // Use default width
            break
    }
    // Resize the image using Sharp
    return sharp(buffer).resize(width).toBuffer()
}

// Export the functions
module.exports = {
    uploadBanner,
    // uploadNews,
    // uploadAnnouncements,
    // uploadComplaints,
    // uploadDococument,
    getMulterStorage,
    processFileOptimization,
    uploadAndOptimize,
}